# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Type, Iterable, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.browser_rendering import link_create_params
from ...types.browser_rendering.link_create_response import LinkCreateResponse

__all__ = ["LinksResource", "AsyncLinksResource"]


class LinksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LinksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LinksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LinksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LinksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[link_create_params.AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[link_create_params.AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: link_create_params.Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[link_create_params.Cookie] | Omit = omit,
        emulate_media_type: str | Omit = omit,
        exclude_external_links: bool | Omit = omit,
        goto_options: link_create_params.GotoOptions | Omit = omit,
        html: str | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        url: str | Omit = omit,
        user_agent: str | Omit = omit,
        viewport: link_create_params.Viewport | Omit = omit,
        visible_links_only: bool | Omit = omit,
        wait_for_selector: link_create_params.WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LinkCreateResponse:
        """
        Get links from a web page.

        Args:
          account_id: Account ID.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          action_timeout: The maximum duration allowed for the browser action to complete after the page
              has loaded (such as taking screenshots, extracting content, or generating PDFs).
              If this time limit is exceeded, the action stops and returns a timeout error.

          add_script_tag: Adds a `<script>` tag into the page with the desired URL or content.

          add_style_tag: Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
              `<style type="text/css">` tag with the content.

          allow_request_pattern: Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'.

          allow_resource_types: Only allow requests that match the provided resource types, eg. 'image' or
              'script'.

          authenticate: Provide credentials for HTTP authentication.

          best_attempt: Attempt to proceed when 'awaited' events fail or timeout.

          cookies: Check [options](https://pptr.dev/api/puppeteer.page.setcookie).

          goto_options: Check [options](https://pptr.dev/api/puppeteer.gotooptions).

          html: Set the content of the page, eg: `<h1>Hello World!!</h1>`. Either `html` or
              `url` must be set.

          reject_request_pattern: Block undesired requests that match the provided regex patterns, eg.
              '/^.\\**\\..(css)'.

          reject_resource_types: Block undesired requests that match the provided resource types, eg. 'image' or
              'script'.

          url: URL to navigate to, eg. `https://example.com`.

          viewport: Check [options](https://pptr.dev/api/puppeteer.page.setviewport).

          wait_for_selector: Wait for the selector to appear in page. Check
              [options](https://pptr.dev/api/puppeteer.page.waitforselector).

          wait_for_timeout: Waits for a specified timeout before continuing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/browser-rendering/links", account_id=account_id),
            body=maybe_transform(
                {
                    "action_timeout": action_timeout,
                    "add_script_tag": add_script_tag,
                    "add_style_tag": add_style_tag,
                    "allow_request_pattern": allow_request_pattern,
                    "allow_resource_types": allow_resource_types,
                    "authenticate": authenticate,
                    "best_attempt": best_attempt,
                    "cookies": cookies,
                    "emulate_media_type": emulate_media_type,
                    "exclude_external_links": exclude_external_links,
                    "goto_options": goto_options,
                    "html": html,
                    "reject_request_pattern": reject_request_pattern,
                    "reject_resource_types": reject_resource_types,
                    "set_extra_http_headers": set_extra_http_headers,
                    "set_java_script_enabled": set_java_script_enabled,
                    "url": url,
                    "user_agent": user_agent,
                    "viewport": viewport,
                    "visible_links_only": visible_links_only,
                    "wait_for_selector": wait_for_selector,
                    "wait_for_timeout": wait_for_timeout,
                },
                link_create_params.LinkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"cache_ttl": cache_ttl}, link_create_params.LinkCreateParams),
                post_parser=ResultWrapper[LinkCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[LinkCreateResponse], ResultWrapper[LinkCreateResponse]),
        )


class AsyncLinksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLinksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLinksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLinksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLinksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[link_create_params.AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[link_create_params.AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: link_create_params.Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[link_create_params.Cookie] | Omit = omit,
        emulate_media_type: str | Omit = omit,
        exclude_external_links: bool | Omit = omit,
        goto_options: link_create_params.GotoOptions | Omit = omit,
        html: str | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        url: str | Omit = omit,
        user_agent: str | Omit = omit,
        viewport: link_create_params.Viewport | Omit = omit,
        visible_links_only: bool | Omit = omit,
        wait_for_selector: link_create_params.WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LinkCreateResponse:
        """
        Get links from a web page.

        Args:
          account_id: Account ID.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          action_timeout: The maximum duration allowed for the browser action to complete after the page
              has loaded (such as taking screenshots, extracting content, or generating PDFs).
              If this time limit is exceeded, the action stops and returns a timeout error.

          add_script_tag: Adds a `<script>` tag into the page with the desired URL or content.

          add_style_tag: Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
              `<style type="text/css">` tag with the content.

          allow_request_pattern: Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'.

          allow_resource_types: Only allow requests that match the provided resource types, eg. 'image' or
              'script'.

          authenticate: Provide credentials for HTTP authentication.

          best_attempt: Attempt to proceed when 'awaited' events fail or timeout.

          cookies: Check [options](https://pptr.dev/api/puppeteer.page.setcookie).

          goto_options: Check [options](https://pptr.dev/api/puppeteer.gotooptions).

          html: Set the content of the page, eg: `<h1>Hello World!!</h1>`. Either `html` or
              `url` must be set.

          reject_request_pattern: Block undesired requests that match the provided regex patterns, eg.
              '/^.\\**\\..(css)'.

          reject_resource_types: Block undesired requests that match the provided resource types, eg. 'image' or
              'script'.

          url: URL to navigate to, eg. `https://example.com`.

          viewport: Check [options](https://pptr.dev/api/puppeteer.page.setviewport).

          wait_for_selector: Wait for the selector to appear in page. Check
              [options](https://pptr.dev/api/puppeteer.page.waitforselector).

          wait_for_timeout: Waits for a specified timeout before continuing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/browser-rendering/links", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "action_timeout": action_timeout,
                    "add_script_tag": add_script_tag,
                    "add_style_tag": add_style_tag,
                    "allow_request_pattern": allow_request_pattern,
                    "allow_resource_types": allow_resource_types,
                    "authenticate": authenticate,
                    "best_attempt": best_attempt,
                    "cookies": cookies,
                    "emulate_media_type": emulate_media_type,
                    "exclude_external_links": exclude_external_links,
                    "goto_options": goto_options,
                    "html": html,
                    "reject_request_pattern": reject_request_pattern,
                    "reject_resource_types": reject_resource_types,
                    "set_extra_http_headers": set_extra_http_headers,
                    "set_java_script_enabled": set_java_script_enabled,
                    "url": url,
                    "user_agent": user_agent,
                    "viewport": viewport,
                    "visible_links_only": visible_links_only,
                    "wait_for_selector": wait_for_selector,
                    "wait_for_timeout": wait_for_timeout,
                },
                link_create_params.LinkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"cache_ttl": cache_ttl}, link_create_params.LinkCreateParams),
                post_parser=ResultWrapper[LinkCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[LinkCreateResponse], ResultWrapper[LinkCreateResponse]),
        )


class LinksResourceWithRawResponse:
    def __init__(self, links: LinksResource) -> None:
        self._links = links

        self.create = to_raw_response_wrapper(
            links.create,
        )


class AsyncLinksResourceWithRawResponse:
    def __init__(self, links: AsyncLinksResource) -> None:
        self._links = links

        self.create = async_to_raw_response_wrapper(
            links.create,
        )


class LinksResourceWithStreamingResponse:
    def __init__(self, links: LinksResource) -> None:
        self._links = links

        self.create = to_streamed_response_wrapper(
            links.create,
        )


class AsyncLinksResourceWithStreamingResponse:
    def __init__(self, links: AsyncLinksResource) -> None:
        self._links = links

        self.create = async_to_streamed_response_wrapper(
            links.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import date

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.organizations.billing import usage_get_params
from ....types.organizations.billing.usage_get_response import UsageGetResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def get(
        self,
        organization_id: str,
        *,
        from_: Union[str, date] | Omit = omit,
        to: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageGetResponse:
        """
        Returns cost and usage data for all accounts within an organization, aligned
        with the [FinOps FOCUS v1.3](https://focus.finops.org/focus-specification/v1-3/)
        Cost and Usage dataset specification.

        Each record represents one billable metric for one account on one day. This
        includes all metered usage, including usage that falls within free-tier
        allowances and may result in zero cost. The response includes usage for every
        account belonging to the specified organization.

        **Note:** Cost and pricing fields are not yet populated and will be absent from
        responses until billing integration is complete.

        When `from` and `to` are omitted, defaults to the start of the current month
        through today. The maximum date range is 31 days.

        Args:
          organization_id: Represents a Cloudflare resource identifier tag.

          from_: Start date for the usage query (ISO 8601). Required if `to` is set. When omitted
              along with `to`, defaults to the start of the current month. Filters by charge
              period (when consumption happened), not billing period. The maximum date range
              is 31 days.

          to: End date for the usage query (ISO 8601). Required if `from` is set. When omitted
              along with `from`, defaults to today. Filters by charge period (when consumption
              happened), not billing period. The maximum date range is 31 days.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not organization_id:
            raise ValueError(f"Expected a non-empty value for `organization_id` but received {organization_id!r}")
        return self._get(
            path_template("/organizations/{organization_id}/billable/usage", organization_id=organization_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                    },
                    usage_get_params.UsageGetParams,
                ),
                post_parser=ResultWrapper[UsageGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[UsageGetResponse], ResultWrapper[UsageGetResponse]),
        )


class AsyncUsageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    async def get(
        self,
        organization_id: str,
        *,
        from_: Union[str, date] | Omit = omit,
        to: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageGetResponse:
        """
        Returns cost and usage data for all accounts within an organization, aligned
        with the [FinOps FOCUS v1.3](https://focus.finops.org/focus-specification/v1-3/)
        Cost and Usage dataset specification.

        Each record represents one billable metric for one account on one day. This
        includes all metered usage, including usage that falls within free-tier
        allowances and may result in zero cost. The response includes usage for every
        account belonging to the specified organization.

        **Note:** Cost and pricing fields are not yet populated and will be absent from
        responses until billing integration is complete.

        When `from` and `to` are omitted, defaults to the start of the current month
        through today. The maximum date range is 31 days.

        Args:
          organization_id: Represents a Cloudflare resource identifier tag.

          from_: Start date for the usage query (ISO 8601). Required if `to` is set. When omitted
              along with `to`, defaults to the start of the current month. Filters by charge
              period (when consumption happened), not billing period. The maximum date range
              is 31 days.

          to: End date for the usage query (ISO 8601). Required if `from` is set. When omitted
              along with `from`, defaults to today. Filters by charge period (when consumption
              happened), not billing period. The maximum date range is 31 days.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not organization_id:
            raise ValueError(f"Expected a non-empty value for `organization_id` but received {organization_id!r}")
        return await self._get(
            path_template("/organizations/{organization_id}/billable/usage", organization_id=organization_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                    },
                    usage_get_params.UsageGetParams,
                ),
                post_parser=ResultWrapper[UsageGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[UsageGetResponse], ResultWrapper[UsageGetResponse]),
        )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.get = to_raw_response_wrapper(
            usage.get,
        )


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.get = async_to_raw_response_wrapper(
            usage.get,
        )


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.get = to_streamed_response_wrapper(
            usage.get,
        )


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.get = async_to_streamed_response_wrapper(
            usage.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .remediation_type_list_params import RemediationTypeListParams as RemediationTypeListParams
from .remediation_type_list_response import RemediationTypeListResponse as RemediationTypeListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .ramp_type import RampType
from ...._models import BaseModel

__all__ = ["Ramp", "GRE", "GREInterconnect", "IPSEC", "Mconn", "MplsInterconnect"]


class GRE(BaseModel):
    managed_by: Optional[str] = None
    """URL reference to the source network resource that this ramp is managed by."""


class GREInterconnect(BaseModel):
    managed_by: Optional[str] = None
    """URL reference to the source network resource that this ramp is managed by."""


class IPSEC(BaseModel):
    managed_by: Optional[str] = None
    """URL reference to the source network resource that this ramp is managed by."""


class Mconn(BaseModel):
    managed_by: Optional[str] = None
    """URL reference to the source network resource that this ramp is managed by."""


class MplsInterconnect(BaseModel):
    managed_by: Optional[str] = None
    """URL reference to the source network resource that this ramp is managed by."""


class Ramp(BaseModel):
    id: str
    """Identifier"""

    created_on: datetime

    modified_on: datetime

    name: str
    """
    A human-provided name describing the ramp that should be unique within the CF1
    Site.
    """

    type: RampType
    """
    The type of network connection (ramp) linking a CF1 Site to Cloudflare's
    network.
    """

    description: Optional[str] = None
    """A human-provided description of the ramp."""

    gre: Optional[GRE] = None

    gre_interconnect: Optional[GREInterconnect] = None

    ipsec: Optional[IPSEC] = None

    mconn: Optional[Mconn] = None

    mpls_interconnect: Optional[MplsInterconnect] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["OriginCloudRegionUpdateParams"]


class OriginCloudRegionUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    body_origin_ip: Required[Annotated[str, PropertyInfo(alias="origin_ip")]]
    """Origin IP address (IPv4 or IPv6).

    For the single PUT endpoint (`PUT /origin/cloud_regions/{origin_ip}`), this
    field must match the path parameter or the request will be rejected with a 400
    error. For the batch PUT endpoint, this field identifies which mapping to
    upsert.
    """

    region: Required[str]
    """Cloud vendor region identifier.

    Must be a valid region for the specified vendor as returned by the
    supported_regions endpoint.
    """

    vendor: Required[Literal["aws", "azure", "gcp", "oci"]]
    """Cloud vendor hosting the origin. Must be one of the supported vendors."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .robots import (
    RobotsResource,
    AsyncRobotsResource,
    RobotsResourceWithRawResponse,
    AsyncRobotsResourceWithRawResponse,
    RobotsResourceWithStreamingResponse,
    AsyncRobotsResourceWithStreamingResponse,
)
from .ai_audit import (
    AIAuditResource,
    AsyncAIAuditResource,
    AIAuditResourceWithRawResponse,
    AsyncAIAuditResourceWithRawResponse,
    AIAuditResourceWithStreamingResponse,
    AsyncAIAuditResourceWithStreamingResponse,
)

__all__ = [
    "RobotsResource",
    "AsyncRobotsResource",
    "RobotsResourceWithRawResponse",
    "AsyncRobotsResourceWithRawResponse",
    "RobotsResourceWithStreamingResponse",
    "AsyncRobotsResourceWithStreamingResponse",
    "AIAuditResource",
    "AsyncAIAuditResource",
    "AIAuditResourceWithRawResponse",
    "AsyncAIAuditResourceWithRawResponse",
    "AIAuditResourceWithStreamingResponse",
    "AsyncAIAuditResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["AnalyticsQuerySummaryResponse"]


class AnalyticsQuerySummaryResponse(BaseModel):
    current_total: List[Dict[str, object]] = FieldInfo(alias="currentTotal")
    """Aggregated stats for the requested time range."""

    previous_total: List[Dict[str, object]] = FieldInfo(alias="previousTotal")
    """Aggregated stats for the equivalent preceding time range, for trend comparison."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "RobotBulkGetResponse",
    "RobotBulkGetResponseItem",
    "RobotBulkGetResponseItemUserAgents",
    "RobotBulkGetResponseItemUserAgentsContentSignals",
]


class RobotBulkGetResponseItemUserAgentsContentSignals(BaseModel):
    """Content signal directives from robots.txt."""

    ai_input: Optional[Literal["yes", "no"]] = FieldInfo(alias="ai-input", default=None)
    """Whether AI input usage is permitted."""

    ai_train: Optional[Literal["yes", "no"]] = FieldInfo(alias="ai-train", default=None)
    """Whether AI training is permitted."""

    search: Optional[Literal["yes", "no"]] = None
    """Whether search indexing is permitted."""


class RobotBulkGetResponseItemUserAgents(BaseModel):
    """Parsed rules for a specific user-agent."""

    allow: List[str]
    """List of allowed path patterns."""

    disallow: List[str]
    """List of disallowed path patterns."""

    content_signals: Optional[RobotBulkGetResponseItemUserAgentsContentSignals] = FieldInfo(
        alias="contentSignals", default=None
    )
    """Content signal directives from robots.txt."""

    crawl_delay: Optional[float] = FieldInfo(alias="crawlDelay", default=None)
    """Crawl delay in seconds."""


class RobotBulkGetResponseItem(BaseModel):
    """Parsed robots.txt rules for a single domain."""

    user_agents: Dict[str, RobotBulkGetResponseItemUserAgents] = FieldInfo(alias="userAgents")
    """Map of user-agent string to its parsed rules."""

    sitemaps: Optional[List[str]] = None
    """List of sitemap URLs found in robots.txt."""

    status: Optional[int] = None
    """HTTP status code from fetching the robots.txt file."""


RobotBulkGetResponse: TypeAlias = Dict[str, RobotBulkGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.radar.quality import speed_summary_params, speed_histogram_params
from .....types.radar.quality.speed_summary_response import SpeedSummaryResponse
from .....types.radar.quality.speed_histogram_response import SpeedHistogramResponse

__all__ = ["SpeedResource", "AsyncSpeedResource"]


class SpeedResource(SyncAPIResource):
    @cached_property
    def top(self) -> TopResource:
        return TopResource(self._client)

    @cached_property
    def with_raw_response(self) -> SpeedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SpeedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SpeedResourceWithStreamingResponse(self)

    def histogram(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        bucket_size: int | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        metric_group: Literal["BANDWIDTH", "LATENCY", "JITTER"] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeedHistogramResponse:
        """
        Retrieves a histogram from the previous 90 days of Cloudflare Speed Test data,
        split into fixed bandwidth (Mbps), latency (ms), or jitter (ms) buckets.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          bucket_size: Specifies the width for every bucket in the histogram.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive). Alternative to `dateRange`; provide together
              with `dateStart`. When requesting comparison series, every series must resolve
              to the same duration as the main series. Each `dateStart`/`dateEnd` is floored
              to the nearest 15 minutes before evaluation, so windows whose durations match
              only before alignment may be rejected.

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          metric_group: Metrics to be returned.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/quality/speed/histogram",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "bucket_size": bucket_size,
                        "continent": continent,
                        "date_end": date_end,
                        "format": format,
                        "location": location,
                        "metric_group": metric_group,
                        "name": name,
                    },
                    speed_histogram_params.SpeedHistogramParams,
                ),
                post_parser=ResultWrapper[SpeedHistogramResponse]._unwrapper,
            ),
            cast_to=cast(Type[SpeedHistogramResponse], ResultWrapper[SpeedHistogramResponse]),
        )

    def summary(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeedSummaryResponse:
        """
        Retrieves a summary of bandwidth, latency, jitter, and packet loss, from the
        previous 90 days of Cloudflare Speed Test data.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive). Alternative to `dateRange`; provide together
              with `dateStart`. When requesting comparison series, every series must resolve
              to the same duration as the main series. Each `dateStart`/`dateEnd` is floored
              to the nearest 15 minutes before evaluation, so windows whose durations match
              only before alignment may be rejected.

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/quality/speed/summary",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "format": format,
                        "location": location,
                        "name": name,
                    },
                    speed_summary_params.SpeedSummaryParams,
                ),
                post_parser=ResultWrapper[SpeedSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[SpeedSummaryResponse], ResultWrapper[SpeedSummaryResponse]),
        )


class AsyncSpeedResource(AsyncAPIResource):
    @cached_property
    def top(self) -> AsyncTopResource:
        return AsyncTopResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSpeedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSpeedResourceWithStreamingResponse(self)

    async def histogram(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        bucket_size: int | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        metric_group: Literal["BANDWIDTH", "LATENCY", "JITTER"] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeedHistogramResponse:
        """
        Retrieves a histogram from the previous 90 days of Cloudflare Speed Test data,
        split into fixed bandwidth (Mbps), latency (ms), or jitter (ms) buckets.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          bucket_size: Specifies the width for every bucket in the histogram.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive). Alternative to `dateRange`; provide together
              with `dateStart`. When requesting comparison series, every series must resolve
              to the same duration as the main series. Each `dateStart`/`dateEnd` is floored
              to the nearest 15 minutes before evaluation, so windows whose durations match
              only before alignment may be rejected.

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          metric_group: Metrics to be returned.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/quality/speed/histogram",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "bucket_size": bucket_size,
                        "continent": continent,
                        "date_end": date_end,
                        "format": format,
                        "location": location,
                        "metric_group": metric_group,
                        "name": name,
                    },
                    speed_histogram_params.SpeedHistogramParams,
                ),
                post_parser=ResultWrapper[SpeedHistogramResponse]._unwrapper,
            ),
            cast_to=cast(Type[SpeedHistogramResponse], ResultWrapper[SpeedHistogramResponse]),
        )

    async def summary(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeedSummaryResponse:
        """
        Retrieves a summary of bandwidth, latency, jitter, and packet loss, from the
        previous 90 days of Cloudflare Speed Test data.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive). Alternative to `dateRange`; provide together
              with `dateStart`. When requesting comparison series, every series must resolve
              to the same duration as the main series. Each `dateStart`/`dateEnd` is floored
              to the nearest 15 minutes before evaluation, so windows whose durations match
              only before alignment may be rejected.

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/quality/speed/summary",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "format": format,
                        "location": location,
                        "name": name,
                    },
                    speed_summary_params.SpeedSummaryParams,
                ),
                post_parser=ResultWrapper[SpeedSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[SpeedSummaryResponse], ResultWrapper[SpeedSummaryResponse]),
        )


class SpeedResourceWithRawResponse:
    def __init__(self, speed: SpeedResource) -> None:
        self._speed = speed

        self.histogram = to_raw_response_wrapper(
            speed.histogram,
        )
        self.summary = to_raw_response_wrapper(
            speed.summary,
        )

    @cached_property
    def top(self) -> TopResourceWithRawResponse:
        return TopResourceWithRawResponse(self._speed.top)


class AsyncSpeedResourceWithRawResponse:
    def __init__(self, speed: AsyncSpeedResource) -> None:
        self._speed = speed

        self.histogram = async_to_raw_response_wrapper(
            speed.histogram,
        )
        self.summary = async_to_raw_response_wrapper(
            speed.summary,
        )

    @cached_property
    def top(self) -> AsyncTopResourceWithRawResponse:
        return AsyncTopResourceWithRawResponse(self._speed.top)


class SpeedResourceWithStreamingResponse:
    def __init__(self, speed: SpeedResource) -> None:
        self._speed = speed

        self.histogram = to_streamed_response_wrapper(
            speed.histogram,
        )
        self.summary = to_streamed_response_wrapper(
            speed.summary,
        )

    @cached_property
    def top(self) -> TopResourceWithStreamingResponse:
        return TopResourceWithStreamingResponse(self._speed.top)


class AsyncSpeedResourceWithStreamingResponse:
    def __init__(self, speed: AsyncSpeedResource) -> None:
        self._speed = speed

        self.histogram = async_to_streamed_response_wrapper(
            speed.histogram,
        )
        self.summary = async_to_streamed_response_wrapper(
            speed.summary,
        )

    @cached_property
    def top(self) -> AsyncTopResourceWithStreamingResponse:
        return AsyncTopResourceWithStreamingResponse(self._speed.top)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.dls import region_list_params
from ...pagination import SyncCursorPagination, AsyncCursorPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.dls.region_get_response import RegionGetResponse
from ...types.dls.region_list_response import RegionListResponse

__all__ = ["RegionsResource", "AsyncRegionsResource"]


class RegionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RegionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RegionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RegionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RegionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        per_page: int | Omit = omit,
        type: Literal["managed", "custom"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPagination[RegionListResponse]:
        """
        List the DLS regions (managed and custom) available to an account.

        Args:
          account_id: Identifier of a Cloudflare account.

          cursor: Opaque token for cursor-based pagination. Omit for the first page. Pass the
              value from a previous response to fetch the next page.

          type: Filter regions by type. Omit to return all regions.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dls/regions", account_id=account_id),
            page=SyncCursorPagination[RegionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "per_page": per_page,
                        "type": type,
                    },
                    region_list_params.RegionListParams,
                ),
            ),
            model=RegionListResponse,
        )

    def get(
        self,
        region_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegionGetResponse:
        """
        Retrieve a single DLS region (managed or custom) by ID or region key.

        Args:
          account_id: Identifier of a Cloudflare account.

          region_id: UUID of the region (custom or managed) or region_key of a managed region.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not region_id:
            raise ValueError(f"Expected a non-empty value for `region_id` but received {region_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/dls/regions/{region_id}", account_id=account_id, region_id=region_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RegionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[RegionGetResponse], ResultWrapper[RegionGetResponse]),
        )


class AsyncRegionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRegionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRegionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRegionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRegionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        per_page: int | Omit = omit,
        type: Literal["managed", "custom"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RegionListResponse, AsyncCursorPagination[RegionListResponse]]:
        """
        List the DLS regions (managed and custom) available to an account.

        Args:
          account_id: Identifier of a Cloudflare account.

          cursor: Opaque token for cursor-based pagination. Omit for the first page. Pass the
              value from a previous response to fetch the next page.

          type: Filter regions by type. Omit to return all regions.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dls/regions", account_id=account_id),
            page=AsyncCursorPagination[RegionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "per_page": per_page,
                        "type": type,
                    },
                    region_list_params.RegionListParams,
                ),
            ),
            model=RegionListResponse,
        )

    async def get(
        self,
        region_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegionGetResponse:
        """
        Retrieve a single DLS region (managed or custom) by ID or region key.

        Args:
          account_id: Identifier of a Cloudflare account.

          region_id: UUID of the region (custom or managed) or region_key of a managed region.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not region_id:
            raise ValueError(f"Expected a non-empty value for `region_id` but received {region_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/dls/regions/{region_id}", account_id=account_id, region_id=region_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RegionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[RegionGetResponse], ResultWrapper[RegionGetResponse]),
        )


class RegionsResourceWithRawResponse:
    def __init__(self, regions: RegionsResource) -> None:
        self._regions = regions

        self.list = to_raw_response_wrapper(
            regions.list,
        )
        self.get = to_raw_response_wrapper(
            regions.get,
        )


class AsyncRegionsResourceWithRawResponse:
    def __init__(self, regions: AsyncRegionsResource) -> None:
        self._regions = regions

        self.list = async_to_raw_response_wrapper(
            regions.list,
        )
        self.get = async_to_raw_response_wrapper(
            regions.get,
        )


class RegionsResourceWithStreamingResponse:
    def __init__(self, regions: RegionsResource) -> None:
        self._regions = regions

        self.list = to_streamed_response_wrapper(
            regions.list,
        )
        self.get = to_streamed_response_wrapper(
            regions.get,
        )


class AsyncRegionsResourceWithStreamingResponse:
    def __init__(self, regions: AsyncRegionsResource) -> None:
        self._regions = regions

        self.list = async_to_streamed_response_wrapper(
            regions.list,
        )
        self.get = async_to_streamed_response_wrapper(
            regions.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = [
    "FindingTuneSeverityResponse",
    "Finding",
    "FindingCategory",
    "FindingRemediation",
    "Integration",
    "IntegrationPolicy",
    "IntegrationVendor",
    "IntegrationZtEnrollment",
    "SeverityOverride",
]


class FindingCategory(BaseModel):
    """Category information for a finding."""

    observation: Literal["Issue", "Insight", "Activity"]
    """The type of the observation."""

    product: Literal["SaaS", "Cloud"]
    """The product category."""

    type: Literal["Content", "Posture"]
    """The type of the finding category."""


class FindingRemediation(BaseModel):
    """Remediation guide information for a finding."""

    id: str
    """Remediation Id."""

    frameworks: List[str]
    """Relevant Compliance Frameworks."""

    guide: str
    """Remediation guide text."""

    impact: str
    """Description of the potential impact."""

    locale: str
    """I18N Locale."""

    threat: str
    """Description of the threat."""


class Finding(BaseModel):
    """Basic finding type information."""

    id: str
    """The unique identifier of the finding."""

    category: FindingCategory
    """Category information for a finding."""

    name: str
    """The name of the finding."""

    severity: Literal["Critical", "High", "Medium", "Low"]
    """The severity level of a finding."""

    vendor: str
    """The SaaS/Cloud vendor of the platform with which the finding is associated."""

    description: Optional[str] = None
    """Detailed description of the finding."""

    remediation: Optional[FindingRemediation] = None
    """Remediation guide information for a finding."""


class IntegrationPolicy(BaseModel):
    """Policy configuration for an integration."""

    id: Optional[str] = None
    """Policy identifier."""

    client_id: Optional[str] = None
    """OAuth client ID for the policy."""

    compliance_level: Optional[str] = None
    """Compliance level for the policy."""

    dlp_enabled: Optional[bool] = None
    """Whether DLP is enabled for this policy."""

    link: Optional[str] = None
    """Link to policy documentation."""

    name: Optional[str] = None
    """Policy name."""

    permissions: Optional[List[str]] = None
    """List of permissions included in the policy."""


class IntegrationVendor(BaseModel):
    """Information about a vendor/service provider."""

    id: str
    """The id of the vendor."""

    description: Optional[str] = None
    """Detailed information about what kinds of issues are detected for this vendor."""

    display_name: str
    """The display name of the vendor."""

    logo: str
    """Logo URL for the vendor."""

    name: str
    """The name of the vendor."""

    static_logo: str
    """Static logo URL for the vendor."""

    zt_enrollments: List[str]
    """The vendor's compatible Zero Trust products."""

    policies: Optional[List[Dict[str, object]]] = None
    """The policies related to the vendor."""


class IntegrationZtEnrollment(BaseModel):
    """Information about a Zero Trust product integration."""

    id: Optional[str] = None
    """The internal identifier of the Zero Trust Product."""

    description: Optional[str] = None
    """Brief description of the Zero Trust Product."""

    display_name: Optional[str] = None
    """The verbose name of the Zero Trust Product."""

    enabled: Optional[bool] = None
    """
    Flag to enable/disable access to the listed integration from the corresponding
    Cloudflare product.
    """


class Integration(BaseModel):
    """Summary information about an integration."""

    created: datetime
    """When entity was created."""

    last_hydrated: datetime
    """When were the integration credentials last updated."""

    name: str
    """Name of the integration."""

    permissions: List[str]
    """The vendor-specific permissions associated with the integration."""

    policy: IntegrationPolicy
    """Policy configuration for an integration."""

    status: str
    """Current status of the integration."""

    updated: datetime
    """Last entity was updated."""

    upgradable: bool
    """Whether the integrations permissions can be updated."""

    vendor: IntegrationVendor
    """Information about a vendor/service provider."""

    zt_enrollments: List[IntegrationZtEnrollment]
    """Zero Trust products associated with this integration."""

    id: Optional[str] = None
    """Integration ID."""

    credential_health_status: Optional[Literal["Initializing", "Healthy", "Unhealthy"]] = None
    """Health status of integration credentials."""

    credentials_expiry: Optional[datetime] = None
    """The date and time when the integration credentials will expire."""

    is_paused: Optional[bool] = None
    """Whether the given integration is paused by the user."""

    upgrade_dismissed: Optional[bool] = None
    """UI State as to whether a potential permissions upgrade has been dismissed."""


class SeverityOverride(BaseModel):
    """Override information for finding severity."""

    created_by: str
    """User ID who created the override."""

    severity: Literal["Critical", "High", "Medium", "Low"]
    """The severity level of a finding."""


class FindingTuneSeverityResponse(BaseModel):
    """Aggregated finding information with counts and metadata.

    This is optimized for list API queries and represents a finding along with its instance statistics.
    """

    id: str
    """Base64 encoded identifier of the security finding."""

    active_count: int
    """Number of active problematic instances identified in the security finding."""

    archived_count: int
    """Number of archived instances identified in the security finding."""

    finding: Finding
    """Basic finding type information."""

    ignored: bool
    """Determines if finding is currently ignored."""

    instance_count: int
    """
    Number of total (Active or archived) problematic instances identified in the
    security finding.
    """

    integration: Integration
    """Summary information about an integration."""

    latest_affliction_date: datetime
    """Timestamp of the latest affliction date of an active finding."""

    severity_override: Optional[SeverityOverride] = None
    """Override information for finding severity."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.dlp import data_class_create_params, data_class_update_params
from ....types.zero_trust.dlp.data_class_get_response import DataClassGetResponse
from ....types.zero_trust.dlp.data_class_list_response import DataClassListResponse
from ....types.zero_trust.dlp.data_class_create_response import DataClassCreateResponse
from ....types.zero_trust.dlp.data_class_update_response import DataClassUpdateResponse

__all__ = ["DataClassesResource", "AsyncDataClassesResource"]


class DataClassesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DataClassesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DataClassesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataClassesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DataClassesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        data_tags: SequenceNotStr[str],
        expression: str,
        name: str,
        sensitivity_levels: Iterable[data_class_create_params.SensitivityLevel],
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataClassCreateResponse]:
        """
        Creates a data class for use in DLP profiles.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/dlp/data_classes", account_id=account_id),
            body=maybe_transform(
                {
                    "data_tags": data_tags,
                    "expression": expression,
                    "name": name,
                    "sensitivity_levels": sensitivity_levels,
                    "description": description,
                },
                data_class_create_params.DataClassCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataClassCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataClassCreateResponse]], ResultWrapper[DataClassCreateResponse]),
        )

    def update(
        self,
        data_class_id: str,
        *,
        account_id: str,
        data_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        expression: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        sensitivity_levels: Optional[Iterable[data_class_update_params.SensitivityLevel]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataClassUpdateResponse]:
        """
        Updates the configuration for a data class.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not data_class_id:
            raise ValueError(f"Expected a non-empty value for `data_class_id` but received {data_class_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/dlp/data_classes/{data_class_id}",
                account_id=account_id,
                data_class_id=data_class_id,
            ),
            body=maybe_transform(
                {
                    "data_tags": data_tags,
                    "description": description,
                    "expression": expression,
                    "name": name,
                    "sensitivity_levels": sensitivity_levels,
                },
                data_class_update_params.DataClassUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataClassUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataClassUpdateResponse]], ResultWrapper[DataClassUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DataClassListResponse]:
        """
        Lists data classes configured for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/data_classes", account_id=account_id),
            page=SyncSinglePage[DataClassListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DataClassListResponse,
        )

    def delete(
        self,
        data_class_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a data class from the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not data_class_id:
            raise ValueError(f"Expected a non-empty value for `data_class_id` but received {data_class_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/dlp/data_classes/{data_class_id}",
                account_id=account_id,
                data_class_id=data_class_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        data_class_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataClassGetResponse]:
        """
        Gets the configuration for a data class.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not data_class_id:
            raise ValueError(f"Expected a non-empty value for `data_class_id` but received {data_class_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dlp/data_classes/{data_class_id}",
                account_id=account_id,
                data_class_id=data_class_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataClassGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataClassGetResponse]], ResultWrapper[DataClassGetResponse]),
        )


class AsyncDataClassesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDataClassesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataClassesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataClassesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDataClassesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        data_tags: SequenceNotStr[str],
        expression: str,
        name: str,
        sensitivity_levels: Iterable[data_class_create_params.SensitivityLevel],
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataClassCreateResponse]:
        """
        Creates a data class for use in DLP profiles.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/dlp/data_classes", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "data_tags": data_tags,
                    "expression": expression,
                    "name": name,
                    "sensitivity_levels": sensitivity_levels,
                    "description": description,
                },
                data_class_create_params.DataClassCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataClassCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataClassCreateResponse]], ResultWrapper[DataClassCreateResponse]),
        )

    async def update(
        self,
        data_class_id: str,
        *,
        account_id: str,
        data_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        expression: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        sensitivity_levels: Optional[Iterable[data_class_update_params.SensitivityLevel]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataClassUpdateResponse]:
        """
        Updates the configuration for a data class.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not data_class_id:
            raise ValueError(f"Expected a non-empty value for `data_class_id` but received {data_class_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/dlp/data_classes/{data_class_id}",
                account_id=account_id,
                data_class_id=data_class_id,
            ),
            body=await async_maybe_transform(
                {
                    "data_tags": data_tags,
                    "description": description,
                    "expression": expression,
                    "name": name,
                    "sensitivity_levels": sensitivity_levels,
                },
                data_class_update_params.DataClassUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataClassUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataClassUpdateResponse]], ResultWrapper[DataClassUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DataClassListResponse, AsyncSinglePage[DataClassListResponse]]:
        """
        Lists data classes configured for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/data_classes", account_id=account_id),
            page=AsyncSinglePage[DataClassListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DataClassListResponse,
        )

    async def delete(
        self,
        data_class_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a data class from the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not data_class_id:
            raise ValueError(f"Expected a non-empty value for `data_class_id` but received {data_class_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dlp/data_classes/{data_class_id}",
                account_id=account_id,
                data_class_id=data_class_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        data_class_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataClassGetResponse]:
        """
        Gets the configuration for a data class.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not data_class_id:
            raise ValueError(f"Expected a non-empty value for `data_class_id` but received {data_class_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dlp/data_classes/{data_class_id}",
                account_id=account_id,
                data_class_id=data_class_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataClassGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataClassGetResponse]], ResultWrapper[DataClassGetResponse]),
        )


class DataClassesResourceWithRawResponse:
    def __init__(self, data_classes: DataClassesResource) -> None:
        self._data_classes = data_classes

        self.create = to_raw_response_wrapper(
            data_classes.create,
        )
        self.update = to_raw_response_wrapper(
            data_classes.update,
        )
        self.list = to_raw_response_wrapper(
            data_classes.list,
        )
        self.delete = to_raw_response_wrapper(
            data_classes.delete,
        )
        self.get = to_raw_response_wrapper(
            data_classes.get,
        )


class AsyncDataClassesResourceWithRawResponse:
    def __init__(self, data_classes: AsyncDataClassesResource) -> None:
        self._data_classes = data_classes

        self.create = async_to_raw_response_wrapper(
            data_classes.create,
        )
        self.update = async_to_raw_response_wrapper(
            data_classes.update,
        )
        self.list = async_to_raw_response_wrapper(
            data_classes.list,
        )
        self.delete = async_to_raw_response_wrapper(
            data_classes.delete,
        )
        self.get = async_to_raw_response_wrapper(
            data_classes.get,
        )


class DataClassesResourceWithStreamingResponse:
    def __init__(self, data_classes: DataClassesResource) -> None:
        self._data_classes = data_classes

        self.create = to_streamed_response_wrapper(
            data_classes.create,
        )
        self.update = to_streamed_response_wrapper(
            data_classes.update,
        )
        self.list = to_streamed_response_wrapper(
            data_classes.list,
        )
        self.delete = to_streamed_response_wrapper(
            data_classes.delete,
        )
        self.get = to_streamed_response_wrapper(
            data_classes.get,
        )


class AsyncDataClassesResourceWithStreamingResponse:
    def __init__(self, data_classes: AsyncDataClassesResource) -> None:
        self._data_classes = data_classes

        self.create = async_to_streamed_response_wrapper(
            data_classes.create,
        )
        self.update = async_to_streamed_response_wrapper(
            data_classes.update,
        )
        self.list = async_to_streamed_response_wrapper(
            data_classes.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            data_classes.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            data_classes.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["TokenDeleteResponse", "Error", "Message"]


class Error(BaseModel):
    code: Optional[int] = None

    message: Optional[str] = None


class Message(BaseModel):
    code: Optional[int] = None

    message: Optional[str] = None


class TokenDeleteResponse(BaseModel):
    errors: List[Error]

    messages: List[Message]

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel
from .cf1_site_location import Cf1SiteLocation

__all__ = ["Cf1Site"]


class Cf1Site(BaseModel):
    name: str
    """
    A human-provided name describing the CF1 Site that should be unique within the
    account.
    """

    id: Optional[str] = None
    """Identifier"""

    created_on: Optional[datetime] = None

    description: Optional[str] = None
    """A human-provided description of the CF1 Site."""

    location: Optional[Cf1SiteLocation] = None

    modified_on: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.cloudforce_one.threat_events import graph_list_params
from ....types.cloudforce_one.threat_events.graph_list_response import GraphListResponse

__all__ = ["GraphResource", "AsyncGraphResource"]


class GraphResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GraphResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return GraphResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GraphResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return GraphResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        direction: str | Omit = omit,
        expand: SequenceNotStr[str] | Omit = omit,
        hydration: str | Omit = omit,
        limit: float | Omit = omit,
        max_nodes: float | Omit = omit,
        relationship_types: SequenceNotStr[str] | Omit = omit,
        seeds: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GraphListResponse:
        """
        Expands the single-level relationship neighborhood of one or more seed nodes
        (event, indicator, or tag) from R2 Data Catalog. Seeds use compact id format
        (type:uuid), e.g. "event:550e8400-...". Multi-seed requests merge and
        deduplicate results server-side. Hydrates neighbor entities with summary data
        from Durable Objects. Supports filtering by relationship type and dataset scope.

        Args:
          account_id: Account ID.

          cursor: Opaque pagination token. Only valid when seeds has exactly 1 entry; 400
              otherwise.

          dataset_ids: Comma-separated dataset UUIDs to restrict neighbor scope. Intersected with ACL
              grants.

          direction: Edge direction relative to each seed: out (seed→neighbors), in (neighbors→seed),
              both (default).

          expand: Comma-separated list of response sections to expand (hydrate). Allowed: `nodes`.
              Omitting `expand` returns identifier-only nodes.

          hydration: Hydration strategy for neighbor nodes when expand=nodes is set. r2_join
              (default): use R2 JOIN query + DO fallback. do_only: use plain R2 query +
              hydrate all neighbors via Durable Objects.

          limit: Max neighbors per seed (default: 100, max: 1000). Values above 1000 return 400.

          max_nodes: Total accumulated node cap across all seeds (default: 500, max: 1000). Values
              above 1000 return 400.

          relationship_types: Comma-separated relationship types to filter by. Allowed: tagged_with,
              appears_in, related_to, caused_by, attributed_to.

          seeds:
              Comma-separated compact seed ids (type:uuid). Example:
              seeds=event:550e8400-...,indicator:661fa920-... Provide 1–50 entries; omitting
              seeds returns 400.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/graph", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "dataset_ids": dataset_ids,
                        "direction": direction,
                        "expand": expand,
                        "hydration": hydration,
                        "limit": limit,
                        "max_nodes": max_nodes,
                        "relationship_types": relationship_types,
                        "seeds": seeds,
                    },
                    graph_list_params.GraphListParams,
                ),
                post_parser=ResultWrapper[GraphListResponse]._unwrapper,
            ),
            cast_to=cast(Type[GraphListResponse], ResultWrapper[GraphListResponse]),
        )


class AsyncGraphResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGraphResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGraphResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGraphResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncGraphResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        direction: str | Omit = omit,
        expand: SequenceNotStr[str] | Omit = omit,
        hydration: str | Omit = omit,
        limit: float | Omit = omit,
        max_nodes: float | Omit = omit,
        relationship_types: SequenceNotStr[str] | Omit = omit,
        seeds: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GraphListResponse:
        """
        Expands the single-level relationship neighborhood of one or more seed nodes
        (event, indicator, or tag) from R2 Data Catalog. Seeds use compact id format
        (type:uuid), e.g. "event:550e8400-...". Multi-seed requests merge and
        deduplicate results server-side. Hydrates neighbor entities with summary data
        from Durable Objects. Supports filtering by relationship type and dataset scope.

        Args:
          account_id: Account ID.

          cursor: Opaque pagination token. Only valid when seeds has exactly 1 entry; 400
              otherwise.

          dataset_ids: Comma-separated dataset UUIDs to restrict neighbor scope. Intersected with ACL
              grants.

          direction: Edge direction relative to each seed: out (seed→neighbors), in (neighbors→seed),
              both (default).

          expand: Comma-separated list of response sections to expand (hydrate). Allowed: `nodes`.
              Omitting `expand` returns identifier-only nodes.

          hydration: Hydration strategy for neighbor nodes when expand=nodes is set. r2_join
              (default): use R2 JOIN query + DO fallback. do_only: use plain R2 query +
              hydrate all neighbors via Durable Objects.

          limit: Max neighbors per seed (default: 100, max: 1000). Values above 1000 return 400.

          max_nodes: Total accumulated node cap across all seeds (default: 500, max: 1000). Values
              above 1000 return 400.

          relationship_types: Comma-separated relationship types to filter by. Allowed: tagged_with,
              appears_in, related_to, caused_by, attributed_to.

          seeds:
              Comma-separated compact seed ids (type:uuid). Example:
              seeds=event:550e8400-...,indicator:661fa920-... Provide 1–50 entries; omitting
              seeds returns 400.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/graph", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "dataset_ids": dataset_ids,
                        "direction": direction,
                        "expand": expand,
                        "hydration": hydration,
                        "limit": limit,
                        "max_nodes": max_nodes,
                        "relationship_types": relationship_types,
                        "seeds": seeds,
                    },
                    graph_list_params.GraphListParams,
                ),
                post_parser=ResultWrapper[GraphListResponse]._unwrapper,
            ),
            cast_to=cast(Type[GraphListResponse], ResultWrapper[GraphListResponse]),
        )


class GraphResourceWithRawResponse:
    def __init__(self, graph: GraphResource) -> None:
        self._graph = graph

        self.list = to_raw_response_wrapper(
            graph.list,
        )


class AsyncGraphResourceWithRawResponse:
    def __init__(self, graph: AsyncGraphResource) -> None:
        self._graph = graph

        self.list = async_to_raw_response_wrapper(
            graph.list,
        )


class GraphResourceWithStreamingResponse:
    def __init__(self, graph: GraphResource) -> None:
        self._graph = graph

        self.list = to_streamed_response_wrapper(
            graph.list,
        )


class AsyncGraphResourceWithStreamingResponse:
    def __init__(self, graph: AsyncGraphResource) -> None:
        self._graph = graph

        self.list = async_to_streamed_response_wrapper(
            graph.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["CategoryListResponse", "Category"]


class Category(BaseModel):
    name: str

    uuid: str

    created_at: Optional[str] = FieldInfo(alias="createdAt", default=None)

    description: Optional[str] = None

    updated_at: Optional[str] = FieldInfo(alias="updatedAt", default=None)


class CategoryListResponse(BaseModel):
    categories: List[Category]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RegistrationEditResponse", "Links", "Error"]


class Links(BaseModel):
    self: str
    """URL to this status resource."""

    resource: Optional[str] = None
    """URL to the domain resource."""


class Error(BaseModel):
    """Error details when a workflow reaches the `failed` state.

    The specific
    error codes and messages depend on the workflow type (registration,
    update, etc.) and the underlying registry response. These workflow
    error codes are separate from immediate HTTP error `errors[].code`
    values returned by non-2xx responses. Surface
    `error.message` to the user for context.
    """

    code: str
    """Machine-readable error code identifying the failure reason."""

    message: str
    """Human-readable explanation of the failure.

    May include registry-specific details.
    """


class RegistrationEditResponse(BaseModel):
    """Status of an async registration workflow."""

    completed: bool
    """Whether the workflow has reached a terminal state.

    `true` when `state` is `succeeded` or `failed`. `false` for `pending`,
    `in_progress`, `action_required`, and `blocked`.
    """

    created_at: datetime

    links: Links

    state: Literal["pending", "in_progress", "action_required", "blocked", "succeeded", "failed"]
    """Workflow lifecycle state.

    - `pending`: Workflow has been created but not yet started processing.
    - `in_progress`: Actively processing. Continue polling `links.self`. The
      workflow has an internal deadline and will not remain in this state
      indefinitely.
    - `action_required`: Paused — requires action by the user (not the system). See
      `context.action` for what is needed. An automated polling loop must break on
      this state; it will not resolve on its own without user intervention.
    - `blocked`: The workflow cannot make progress due to a third party such as the
      domain extension's registry or a losing registrar. No user action will help.
      Continue polling — the block may resolve when the third party responds.
    - `succeeded`: Terminal. The operation completed successfully. `completed` will
      be `true`. For registrations, `context.registration` contains the resulting
      registration resource.
    - `failed`: Terminal. The operation failed. `completed` will be `true`. See
      `error.code` and `error.message` for the reason. Do not auto-retry without
      user review.
    """

    updated_at: datetime

    context: Optional[Dict[str, object]] = None
    """Workflow-specific data for this workflow.

    The workflow subject is identified by `context.domain_name` for domain-centric
    workflows.
    """

    error: Optional[Error] = None
    """Error details when a workflow reaches the `failed` state.

    The specific error codes and messages depend on the workflow type (registration,
    update, etc.) and the underlying registry response. These workflow error codes
    are separate from immediate HTTP error `errors[].code` values returned by
    non-2xx responses. Surface `error.message` to the user for context.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .spf.spf import (
    SPFResource,
    AsyncSPFResource,
    SPFResourceWithRawResponse,
    AsyncSPFResourceWithRawResponse,
    SPFResourceWithStreamingResponse,
    AsyncSPFResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .dmarc_reports import (
    DMARCReportsResource,
    AsyncDMARCReportsResource,
    DMARCReportsResourceWithRawResponse,
    AsyncDMARCReportsResourceWithRawResponse,
    DMARCReportsResourceWithStreamingResponse,
    AsyncDMARCReportsResourceWithStreamingResponse,
)

__all__ = ["EmailAuthResource", "AsyncEmailAuthResource"]


class EmailAuthResource(SyncAPIResource):
    @cached_property
    def dmarc_reports(self) -> DMARCReportsResource:
        return DMARCReportsResource(self._client)

    @cached_property
    def spf(self) -> SPFResource:
        return SPFResource(self._client)

    @cached_property
    def with_raw_response(self) -> EmailAuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EmailAuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmailAuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EmailAuthResourceWithStreamingResponse(self)


class AsyncEmailAuthResource(AsyncAPIResource):
    @cached_property
    def dmarc_reports(self) -> AsyncDMARCReportsResource:
        return AsyncDMARCReportsResource(self._client)

    @cached_property
    def spf(self) -> AsyncSPFResource:
        return AsyncSPFResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEmailAuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmailAuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmailAuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEmailAuthResourceWithStreamingResponse(self)


class EmailAuthResourceWithRawResponse:
    def __init__(self, email_auth: EmailAuthResource) -> None:
        self._email_auth = email_auth

    @cached_property
    def dmarc_reports(self) -> DMARCReportsResourceWithRawResponse:
        return DMARCReportsResourceWithRawResponse(self._email_auth.dmarc_reports)

    @cached_property
    def spf(self) -> SPFResourceWithRawResponse:
        return SPFResourceWithRawResponse(self._email_auth.spf)


class AsyncEmailAuthResourceWithRawResponse:
    def __init__(self, email_auth: AsyncEmailAuthResource) -> None:
        self._email_auth = email_auth

    @cached_property
    def dmarc_reports(self) -> AsyncDMARCReportsResourceWithRawResponse:
        return AsyncDMARCReportsResourceWithRawResponse(self._email_auth.dmarc_reports)

    @cached_property
    def spf(self) -> AsyncSPFResourceWithRawResponse:
        return AsyncSPFResourceWithRawResponse(self._email_auth.spf)


class EmailAuthResourceWithStreamingResponse:
    def __init__(self, email_auth: EmailAuthResource) -> None:
        self._email_auth = email_auth

    @cached_property
    def dmarc_reports(self) -> DMARCReportsResourceWithStreamingResponse:
        return DMARCReportsResourceWithStreamingResponse(self._email_auth.dmarc_reports)

    @cached_property
    def spf(self) -> SPFResourceWithStreamingResponse:
        return SPFResourceWithStreamingResponse(self._email_auth.spf)


class AsyncEmailAuthResourceWithStreamingResponse:
    def __init__(self, email_auth: AsyncEmailAuthResource) -> None:
        self._email_auth = email_auth

    @cached_property
    def dmarc_reports(self) -> AsyncDMARCReportsResourceWithStreamingResponse:
        return AsyncDMARCReportsResourceWithStreamingResponse(self._email_auth.dmarc_reports)

    @cached_property
    def spf(self) -> AsyncSPFResourceWithStreamingResponse:
        return AsyncSPFResourceWithStreamingResponse(self._email_auth.spf)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, Iterable, cast
from datetime import datetime

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.analytics_query.data_security import finding_summary_params, finding_timeseries_params
from ....types.analytics_query.data_security.finding_summary_response import FindingSummaryResponse
from ....types.analytics_query.data_security.finding_timeseries_response import FindingTimeseriesResponse

__all__ = ["FindingsResource", "AsyncFindingsResource"]


class FindingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FindingsResourceWithStreamingResponse(self)

    def summary(
        self,
        *,
        account_id: str,
        filters: Iterable[finding_summary_params.Filter],
        from_: Union[str, datetime],
        to: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FindingSummaryResponse:
        """
        Returns aggregate current-period and previous-period totals for CASB findings.

        Args:
          filters: Filters to apply.

          from_: Start of the query time range (inclusive). RFC3339.

          to: End of the query time range (exclusive). RFC3339.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/analytics/query/data-security/findings/summary", account_id=account_id
            ),
            body=maybe_transform(
                {
                    "filters": filters,
                    "from_": from_,
                    "to": to,
                },
                finding_summary_params.FindingSummaryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FindingSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[FindingSummaryResponse], ResultWrapper[FindingSummaryResponse]),
        )

    def timeseries(
        self,
        *,
        account_id: str,
        filters: Iterable[finding_timeseries_params.Filter],
        from_: Union[str, datetime],
        to: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FindingTimeseriesResponse:
        """
        Returns merged time-bucketed CASB findings.

        Args:
          filters: Filters to apply.

          from_: Start of the query time range (inclusive). RFC3339.

          to: End of the query time range (exclusive). RFC3339.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/analytics/query/data-security/findings/timeseries", account_id=account_id
            ),
            body=maybe_transform(
                {
                    "filters": filters,
                    "from_": from_,
                    "to": to,
                },
                finding_timeseries_params.FindingTimeseriesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FindingTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[FindingTimeseriesResponse], ResultWrapper[FindingTimeseriesResponse]),
        )


class AsyncFindingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFindingsResourceWithStreamingResponse(self)

    async def summary(
        self,
        *,
        account_id: str,
        filters: Iterable[finding_summary_params.Filter],
        from_: Union[str, datetime],
        to: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FindingSummaryResponse:
        """
        Returns aggregate current-period and previous-period totals for CASB findings.

        Args:
          filters: Filters to apply.

          from_: Start of the query time range (inclusive). RFC3339.

          to: End of the query time range (exclusive). RFC3339.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/analytics/query/data-security/findings/summary", account_id=account_id
            ),
            body=await async_maybe_transform(
                {
                    "filters": filters,
                    "from_": from_,
                    "to": to,
                },
                finding_summary_params.FindingSummaryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FindingSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[FindingSummaryResponse], ResultWrapper[FindingSummaryResponse]),
        )

    async def timeseries(
        self,
        *,
        account_id: str,
        filters: Iterable[finding_timeseries_params.Filter],
        from_: Union[str, datetime],
        to: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FindingTimeseriesResponse:
        """
        Returns merged time-bucketed CASB findings.

        Args:
          filters: Filters to apply.

          from_: Start of the query time range (inclusive). RFC3339.

          to: End of the query time range (exclusive). RFC3339.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/analytics/query/data-security/findings/timeseries", account_id=account_id
            ),
            body=await async_maybe_transform(
                {
                    "filters": filters,
                    "from_": from_,
                    "to": to,
                },
                finding_timeseries_params.FindingTimeseriesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FindingTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[FindingTimeseriesResponse], ResultWrapper[FindingTimeseriesResponse]),
        )


class FindingsResourceWithRawResponse:
    def __init__(self, findings: FindingsResource) -> None:
        self._findings = findings

        self.summary = to_raw_response_wrapper(
            findings.summary,
        )
        self.timeseries = to_raw_response_wrapper(
            findings.timeseries,
        )


class AsyncFindingsResourceWithRawResponse:
    def __init__(self, findings: AsyncFindingsResource) -> None:
        self._findings = findings

        self.summary = async_to_raw_response_wrapper(
            findings.summary,
        )
        self.timeseries = async_to_raw_response_wrapper(
            findings.timeseries,
        )


class FindingsResourceWithStreamingResponse:
    def __init__(self, findings: FindingsResource) -> None:
        self._findings = findings

        self.summary = to_streamed_response_wrapper(
            findings.summary,
        )
        self.timeseries = to_streamed_response_wrapper(
            findings.timeseries,
        )


class AsyncFindingsResourceWithStreamingResponse:
    def __init__(self, findings: AsyncFindingsResource) -> None:
        self._findings = findings

        self.summary = async_to_streamed_response_wrapper(
            findings.summary,
        )
        self.timeseries = async_to_streamed_response_wrapper(
            findings.timeseries,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .levels.levels import (
    LevelsResource,
    AsyncLevelsResource,
    LevelsResourceWithRawResponse,
    AsyncLevelsResourceWithRawResponse,
    LevelsResourceWithStreamingResponse,
    AsyncLevelsResourceWithStreamingResponse,
)
from .....pagination import SyncSinglePage, AsyncSinglePage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.dlp import sensitivity_group_create_params, sensitivity_group_update_params
from .....types.zero_trust.dlp.sensitivity_group_get_response import SensitivityGroupGetResponse
from .....types.zero_trust.dlp.sensitivity_group_list_response import SensitivityGroupListResponse
from .....types.zero_trust.dlp.sensitivity_group_create_response import SensitivityGroupCreateResponse
from .....types.zero_trust.dlp.sensitivity_group_update_response import SensitivityGroupUpdateResponse

__all__ = ["SensitivityGroupsResource", "AsyncSensitivityGroupsResource"]


class SensitivityGroupsResource(SyncAPIResource):
    @cached_property
    def levels(self) -> LevelsResource:
        return LevelsResource(self._client)

    @cached_property
    def with_raw_response(self) -> SensitivityGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SensitivityGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SensitivityGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SensitivityGroupsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        description: Optional[str] | Omit = omit,
        levels: Iterable[sensitivity_group_create_params.Level] | Omit = omit,
        template_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SensitivityGroupCreateResponse]:
        """
        Creates a sensitivity group, optionally from a template.

        Args:
          levels: Levels to create with the group. Mutually exclusive with `template_id`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/dlp/sensitivity_groups", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "levels": levels,
                    "template_id": template_id,
                },
                sensitivity_group_create_params.SensitivityGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SensitivityGroupCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SensitivityGroupCreateResponse]], ResultWrapper[SensitivityGroupCreateResponse]),
        )

    def update(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        description: Optional[str] | Omit = omit,
        levels: Optional[Iterable[sensitivity_group_update_params.Level]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SensitivityGroupUpdateResponse]:
        """
        Updates a sensitivity group and its levels.

        Args:
          levels: The desired final state of levels.

              - `None` (omitted): no level changes.
              - `Some([])`: delete all levels.
              - `Some([...])`: desired final set + order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return self._put(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "levels": levels,
                    "name": name,
                },
                sensitivity_group_update_params.SensitivityGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SensitivityGroupUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SensitivityGroupUpdateResponse]], ResultWrapper[SensitivityGroupUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[SensitivityGroupListResponse]:
        """
        Lists sensitivity groups configured for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/sensitivity_groups", account_id=account_id),
            page=SyncSinglePage[SensitivityGroupListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=SensitivityGroupListResponse,
        )

    def delete(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a sensitivity group and its levels.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return self._delete(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SensitivityGroupGetResponse]:
        """
        Gets a sensitivity group and its levels.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return self._get(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SensitivityGroupGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SensitivityGroupGetResponse]], ResultWrapper[SensitivityGroupGetResponse]),
        )


class AsyncSensitivityGroupsResource(AsyncAPIResource):
    @cached_property
    def levels(self) -> AsyncLevelsResource:
        return AsyncLevelsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSensitivityGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSensitivityGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSensitivityGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSensitivityGroupsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        description: Optional[str] | Omit = omit,
        levels: Iterable[sensitivity_group_create_params.Level] | Omit = omit,
        template_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SensitivityGroupCreateResponse]:
        """
        Creates a sensitivity group, optionally from a template.

        Args:
          levels: Levels to create with the group. Mutually exclusive with `template_id`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/dlp/sensitivity_groups", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "levels": levels,
                    "template_id": template_id,
                },
                sensitivity_group_create_params.SensitivityGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SensitivityGroupCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SensitivityGroupCreateResponse]], ResultWrapper[SensitivityGroupCreateResponse]),
        )

    async def update(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        description: Optional[str] | Omit = omit,
        levels: Optional[Iterable[sensitivity_group_update_params.Level]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SensitivityGroupUpdateResponse]:
        """
        Updates a sensitivity group and its levels.

        Args:
          levels: The desired final state of levels.

              - `None` (omitted): no level changes.
              - `Some([])`: delete all levels.
              - `Some([...])`: desired final set + order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return await self._put(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "levels": levels,
                    "name": name,
                },
                sensitivity_group_update_params.SensitivityGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SensitivityGroupUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SensitivityGroupUpdateResponse]], ResultWrapper[SensitivityGroupUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SensitivityGroupListResponse, AsyncSinglePage[SensitivityGroupListResponse]]:
        """
        Lists sensitivity groups configured for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/sensitivity_groups", account_id=account_id),
            page=AsyncSinglePage[SensitivityGroupListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=SensitivityGroupListResponse,
        )

    async def delete(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a sensitivity group and its levels.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SensitivityGroupGetResponse]:
        """
        Gets a sensitivity group and its levels.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return await self._get(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SensitivityGroupGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SensitivityGroupGetResponse]], ResultWrapper[SensitivityGroupGetResponse]),
        )


class SensitivityGroupsResourceWithRawResponse:
    def __init__(self, sensitivity_groups: SensitivityGroupsResource) -> None:
        self._sensitivity_groups = sensitivity_groups

        self.create = to_raw_response_wrapper(
            sensitivity_groups.create,
        )
        self.update = to_raw_response_wrapper(
            sensitivity_groups.update,
        )
        self.list = to_raw_response_wrapper(
            sensitivity_groups.list,
        )
        self.delete = to_raw_response_wrapper(
            sensitivity_groups.delete,
        )
        self.get = to_raw_response_wrapper(
            sensitivity_groups.get,
        )

    @cached_property
    def levels(self) -> LevelsResourceWithRawResponse:
        return LevelsResourceWithRawResponse(self._sensitivity_groups.levels)


class AsyncSensitivityGroupsResourceWithRawResponse:
    def __init__(self, sensitivity_groups: AsyncSensitivityGroupsResource) -> None:
        self._sensitivity_groups = sensitivity_groups

        self.create = async_to_raw_response_wrapper(
            sensitivity_groups.create,
        )
        self.update = async_to_raw_response_wrapper(
            sensitivity_groups.update,
        )
        self.list = async_to_raw_response_wrapper(
            sensitivity_groups.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sensitivity_groups.delete,
        )
        self.get = async_to_raw_response_wrapper(
            sensitivity_groups.get,
        )

    @cached_property
    def levels(self) -> AsyncLevelsResourceWithRawResponse:
        return AsyncLevelsResourceWithRawResponse(self._sensitivity_groups.levels)


class SensitivityGroupsResourceWithStreamingResponse:
    def __init__(self, sensitivity_groups: SensitivityGroupsResource) -> None:
        self._sensitivity_groups = sensitivity_groups

        self.create = to_streamed_response_wrapper(
            sensitivity_groups.create,
        )
        self.update = to_streamed_response_wrapper(
            sensitivity_groups.update,
        )
        self.list = to_streamed_response_wrapper(
            sensitivity_groups.list,
        )
        self.delete = to_streamed_response_wrapper(
            sensitivity_groups.delete,
        )
        self.get = to_streamed_response_wrapper(
            sensitivity_groups.get,
        )

    @cached_property
    def levels(self) -> LevelsResourceWithStreamingResponse:
        return LevelsResourceWithStreamingResponse(self._sensitivity_groups.levels)


class AsyncSensitivityGroupsResourceWithStreamingResponse:
    def __init__(self, sensitivity_groups: AsyncSensitivityGroupsResource) -> None:
        self._sensitivity_groups = sensitivity_groups

        self.create = async_to_streamed_response_wrapper(
            sensitivity_groups.create,
        )
        self.update = async_to_streamed_response_wrapper(
            sensitivity_groups.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sensitivity_groups.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sensitivity_groups.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            sensitivity_groups.get,
        )

    @cached_property
    def levels(self) -> AsyncLevelsResourceWithStreamingResponse:
        return AsyncLevelsResourceWithStreamingResponse(self._sensitivity_groups.levels)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .csam_scanner_edit_params import CsamScannerEditParams as CsamScannerEditParams
from .csam_scanner_get_response import CsamScannerGetResponse as CsamScannerGetResponse
from .csam_scanner_edit_response import CsamScannerEditResponse as CsamScannerEditResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .origin_tls_compliance_mode_edit_params import (
    OriginTLSComplianceModeEditParams as OriginTLSComplianceModeEditParams,
)
from .origin_tls_compliance_mode_get_response import (
    OriginTLSComplianceModeGetResponse as OriginTLSComplianceModeGetResponse,
)
from .origin_tls_compliance_mode_edit_response import (
    OriginTLSComplianceModeEditResponse as OriginTLSComplianceModeEditResponse,
)
from .origin_tls_compliance_mode_update_params import (
    OriginTLSComplianceModeUpdateParams as OriginTLSComplianceModeUpdateParams,
)
from .origin_tls_compliance_mode_delete_response import (
    OriginTLSComplianceModeDeleteResponse as OriginTLSComplianceModeDeleteResponse,
)
from .origin_tls_compliance_mode_update_response import (
    OriginTLSComplianceModeUpdateResponse as OriginTLSComplianceModeUpdateResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["HoldEditParams"]


class HoldEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    hold_after: Optional[str]
    """
    If `hold_after` is provided and future-dated, the hold will be temporarily
    disabled, then automatically re-enabled by the system at the time specified in
    this RFC3339-formatted timestamp. A past-dated `hold_after` value will have no
    effect on an existing, enabled hold. Providing an empty string will set its
    value to the current time. Providing `null` will disable the hold indefinitely.
    """

    include_subdomains: bool
    """
    If `true`, the zone hold will extend to block any subdomain of the given zone,
    as well as SSL4SaaS Custom Hostnames. For example, a zone hold on a zone with
    the hostname 'example.com' and include_subdomains=true will block 'example.com',
    'staging.example.com', 'api.staging.example.com', etc.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["InterruptCreateResponse", "Reboot", "Restart", "Shutdown"]


class Reboot(BaseModel):
    purge: Optional[bool] = None
    """Purge connector state."""


class Restart(BaseModel):
    purge: Optional[bool] = None
    """Purge connector state."""


class Shutdown(BaseModel):
    purge: Optional[bool] = None
    """Purge connector state."""


class InterruptCreateResponse(BaseModel):
    """Interrupt action for a connector."""

    submitted_at: str

    reboot: Optional[Reboot] = None

    restart: Optional[Restart] = None

    shutdown: Optional[Shutdown] = None

    triggered_at: Optional[str] = None
